<?php
namespace Teknisa\CMP\Services;

use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class ImportacaoAfTxtService {
  
  protected $entityManager;
  protected $connection;
  protected $newCode;
    
  public function __construct($entityManager, $environment, $newCode) {
    $this->entityManager  = $entityManager;
    $this->connection     = $entityManager->getConnection();
    $this->environment    = $environment;
    $this->newCode        = $newCode;
  }
   
  public function iniciaConexao() {
    $this->connection->beginTransaction();
  }
  
  public function realizaCommit() {
    $this->connection->commit();
  }

  public function realizaRollback() {
    $this->connection->rollBack();
  }
  
  public function executaSql($sql, $params, $consulta = true) {
    try{
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        if($consulta == true) {
            return $std->fetchAll();
        }
        return true;
    } catch (\Exception $e){
        throw $e;
    } 
  }
  
  public function selectNRAUTOFORM () {
    $select = "SELECT SEQ_NRAUTOFORNE_MOBILE.NEXTVAL P_NRAUTOFORNE_MOBILE FROM DUAL";
    return self::executaSql($select, false);
  }
  
  public function insertItens($dataNRAUTOFORNE, $itemParam) {
    $insert = "INSERT 
              INTO AUTOFORNE_MOBILE(NRAUTOFORNE_MOBILE, NRLINHA, CDOPERADOR, CDFORNECED, CDFILDEST, CDPRODUTO, CDMARCA, DTPROGENTAF, QTPROGENTAF, VRPREUNIAF, USR_NRAUTOFORN)
              VALUES (:P_NRAUTOFORNE_MOBILE, :P_NRLINHA, :P_CDOPERADOR, :P_CDFORNECED, :P_CDFILDEST, :P_CDPRODUTO, :P_CDMARCA, :P_DTPROGENTAF, :P_QTPROGENTAF, :P_VRPREUNIAF, :P_USR_NRAUTOFORN)";
    $params = [
      'P_NRAUTOFORNE_MOBILE' => $dataNRAUTOFORNE['P_NRAUTOFORNE_MOBILE'],
      'P_NRLINHA' => $itemParam['VRLINHA'],
      'P_CDOPERADOR' => $itemParam['CDOPERADOR'],
      'P_CDFORNECED' => $itemParam['NRINSJURFORN'],
      'P_CDFILDEST' => $itemParam['CDFILDEST'],
      'P_CDPRODUTO' => $itemParam['CDARVPROD'],
      'P_CDMARCA' => $itemParam['CDMARCA'],
      'P_DTPROGENTAF' => $itemParam['DTPROGENTAF'],
      'P_QTPROGENTAF' => $itemParam['QTPROGENTAF'],
      'P_VRPREUNIAF' => $itemParam['VRPREUNIAF'],
      'P_USR_NRAUTOFORN' => $itemParam['URS_NRAUTOFORN']
    ];
    return self::executaSql($insert, $params, false);
  }
  
  public function selectVD_AUTOFORNE_MOBILE($dataNRAUTOFORNE, $itemParam) {
    $delete = "SELECT NRLINHA, POSICAO, TAMANHO, MENSERRO, VALOR 
              FROM TABLE (VD_AUTOFORNE_MOBILE (:P_NRAUTOFORNE_MOBILE, :P_CDFILDEST))";
    $params = [
      'P_NRAUTOFORNE_MOBILE' => $dataNRAUTOFORNE['P_NRAUTOFORNE_MOBILE'],
      'P_CDFILDEST' => $itemParam['CDFILDEST']
    ];
    return self::executaSql($delete, $params, true);
  }
  
  public function deleteAUTOFORNEMOBILE ($dataNRAUTOFORNE) {
    $delete = "DELETE 
              FROM AUTOFORNE_MOBILE
              WHERE NRAUTOFORNE_MOBILE = :P_NRAUTOFORNE_MOBILE";
    $params = [
      'P_NRAUTOFORNE_MOBILE' => $dataNRAUTOFORNE['P_NRAUTOFORNE_MOBILE']
    ];
    return self::executaSql($delete, $params, false);
  }
  
  public function procedureIMP_AUTOFORNE_MOBILE ($dataNRAUTOFORNE, $itemParam) {
    $procedure = new StoredProcedure($this->connection, 'IMP_AUTOFORNE_MOBILE');
    $procedure->addParam(new Param('P_NRAUTOFORNE_MOBILE', Param::PARAM_INPUT, $dataNRAUTOFORNE['P_NRAUTOFORNE_MOBILE'], Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, $itemParam['P_CDFILLANC'], Param::PARAM_TYPE_STR));
    $response = $procedure->execute();
    return $response;
  }

}